	.include	"lib/global.h"

	.area	HEADER	(ABS)

	.org	.MODE_TABLE+4*.T_MODE
	JP		.tmode

	.module	Terminal

	.area	_BSS

.winulx:
	.ds		0x01
.winuly:
	.ds		0x01
.winlrx:
	.ds		0x01
.winlry:
	.ds		0x01
.curx:
	.ds		0x01
.cury:
	.ds		0x01
.bright:
	.ds		0x01
.kpadx:
	.ds		0x01
.kpady:
	.ds		0x01
.color:
	.ds		0x01
.inlen:
	.ds		0x01
.inbuf:
	.ds		0x20

	.area	_CODE

	.BS		=	0x08
	.CR		=	0x0D

	; Enter text mode
.tmode::
	CALL	.waitvbl		; Must be in VBL before turning the screen off.

	LD		A,#0b00010001	; LCD Controller = Off (no picture on screen)
	LDH		(.LCDCONT),A

;	WindowBank	=	0x9800 (Not used)
;	Window		=	Off
;	BG Chr		=	0x8000
;	BG Bank		=	0x9800
;	OBJ			=	8x8
;	OBJ			=	Off
;	BG			=	On

	CALL	.std_pal		; Normal palette

	CALL 	.mv_char		; Move the charset to 0x8000

	LD		BC,#.splash
	CALL	.set_btt		; Move the text to 0x9800

	LD		A,#0b10010001
	LDH		(.LCDCONT),A

	LD		A,#0x13
	LD		(.inlen),A

	LD		B,#0x01			; Set window for whole screen
	LD		C,#0x01
	LD		D,#0x14
	LD		E,#0x12
	CALL	.window

	LD		B,#.START
	CALL	.waitpad		; Hold startup screen until start is pressed
	CALL	.padup			; Wait until key is released
	CALL	.cls

	CALL	.initkpd

	LD		B,#0x01
	LD		C,#0x01
	CALL	.locate
	CALL	.highlight

	RET

	; Initialize the tile set with characters
.mv_char::
	LD		D,#0x00			; Like move 1024 bytes man
	LD		E,#0x04			; x2 = 1024

	LD		HL,#0x8000
	LD		BC,#.chset
1$:
	LD		A,(BC)
	LD		(HL+),A
	LD		(HL+),A
	INC		BC
	DEC		D
	JR		NZ,1$
	DEC		E
	JR		NZ,1$

	LD		HL,#0x8800		; Fill second set with same at half brightness
	LD		BC,#.chset
	LD		D,#0x00			; Like move 1024 bytes man
	LD		E,#0x04			; x2 = 1024

2$:
	LD		A,(BC)
	LD		(HL),A
	INC		HL
	LD		(HL),#0x00
	INC		HL
	INC		BC
	DEC		D
	JR		NZ,2$
	DEC		E
	JR		NZ,2$
	RET

	; Get a key from Input device & keep most register values
.getch:
	PUSH	BC
	PUSH	DE
	PUSH	HL
	CALL	1$
	POP		HL
	POP		DE
	POP		BC
	RET

	; Get a key from Input device
1$:
	CALL	.jpad
	LD		B,A
	AND		#.SELECT		; Is select pressed ?
	JR		NZ,8$			; Yes, return what keypad's pointing to

	LD		A,B
	AND		#.B				; Is B pressed ?
	LD		A,#.BS
	JP		NZ,.padup		; Yes, backspace

	LD		A,B
	AND		#.START			; Is start pressed ?
	LD		A,#.CR
	JP		NZ,.padup		; Yes

	LD		A,B
	AND		#0x0F
	ADD		A,#0xF0+.DOWN	; Is down pressed ?
	JR		NC,2$			; No

	LD		A,(.kpady)
	XOR		#0x03
	LD		(.kpady),A
	JR		7$

2$:
	ADD		A,#.UP			; Is up pressed ?
	JR		NC,3$			; No

	LD		A,(.kpady)
	XOR		#0x03
	LD		(.kpady),A
	JR		7$

3$:
	ADD		A,#.LEFT		; Is left pressed ?
	JR		NC,5$			; No

	LD		A,(.kpadx)
	CP		#0x01			; Are we already at left column ?
	JR		Z,4$			; Yes

	DEC		A
	LD		(.kpadx),A

	JR		7$
4$:
	LD		A,#0x14
	LD		(.kpadx),A
	JR		7$

5$:
	ADD		A,#.RIGHT		; Is right pressed?
	JR		NC,1$			; No

	LD		A,(.kpadx)
	CP		#0x14			; Are we already at right column?
	JR		Z,6$			; Yes

	INC		A
	LD		(.kpadx),A
	JR		7$

6$:	LD		A,#0x01
	LD		(.kpadx),A
7$:
	CALL	.drawmenu
	CALL	.padup
	JR		1$

8$:
	CALL	.padup
	JP		.xy2char

	; Convert B,C keypad character to actual value
.xy2char:
	LD		DE,#.menu
	LD		C,#0x01
1$:
	LD		B,#0x01
2$:
	LD		A,(.kpadx)
	CP		B
	JR		NZ,4$
	LD		A,(.kpady)
	CP		C
	JR		NZ,4$
	LD		A,(DE)
	CP		#0x00			; Was space pressed ?
	JR		NZ,3$			; No
	LD		A,#' '
3$:
	RET
4$:
	INC		DE
	INC		B
	LD		A,#0x15
	CP		B
	JR		NZ,2$
	INC		C
	JR		1$

; Initialize on screen keypad
.initkpd:
	LD		A,#0x01
	LD		(.kpadx),A
	LD		(.kpady),A

	CALL	.drawmenu

	CALL	.highlight

	LD		B,#0x01			; Set window for all but menu
	LD		C,#0x01
	LD		D,#0x14
	LD		E,#0x10
	JP		.window

; Draw Menu
.drawmenu:
	LD		A,(.curx)
	LD		B,A
	LD		A,(.cury)
	LD		C,A
	PUSH	BC

	LD		B,#0x01
	LD		C,#0x11
	CALL	.locate

	LD		HL,#.menu
	LD		C,#0x01
1$:
	LD		B,#0x01
2$:
	LD		A,(HL+)
	OR		A
	JR		Z,4$

	PUSH	AF
	CALL	.lowlight
	LD		A,(.kpadx)
	CP		B
	JR		NZ,3$
	LD		A,(.kpady)
	CP		C
	JR		NZ,3$

	LD		D,B
	LD		E,C

	CALL	.highlight
3$:
	POP	AF
	CALL	.outchar
	INC		B
	LD		A,#0x15
	CP		B
	JR		NZ,2$
	INC		C
	JR		1$
4$:
	CALL	.lowlight
	LD		A,#' '
	CALL	.outch
	CALL	.highlight
	POP		BC
	LD		A,B
	LD		(.curx),A
	LD		A,C
	LD		(.cury),A
	RET

.menu:
	.ascii	"abcdefghijklmnopqrst"
	.ascii	"uvwxyz0123456789,()"
	.byte	0x00

.locate:
	LD		A,B
	LD		(.curx),A
	LD		A,C
	LD		(.cury),A
	RET

	; Clear Screen
.cls:
	LD		A,#0x12
1$:
	PUSH	AF
	CALL	.scroll
	POP		AF
	DEC		A
	JR		NZ,1$
	RET

	; Scroll Window up
	; Upper left coordinate of window is (.winulx,.winuly).
	; Lower right coordinate of window is (.winlrx,.winlry).
	; Possible Values are (0,0) - (19,17)
.scroll:
	LD		A,(.winuly)
	LD		L,A
	LD		H,#0x00
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	LD		A,(.winulx)
	LD		C,A
	LD		B,#0x00
	ADD		HL,BC
	LD		BC,#0x9800
	ADD		HL,BC
	PUSH	HL				; Put UL address on stack
	LD		BC,#0x20
	ADD		HL,BC			; HL = next line
	POP		BC

	LD		A,(.winuly)
	LD		E,A
	LD		A,(.winlry)
	SUB		E
	LD		E,A				; E = window heigth in characters

1$:
	PUSH	BC

	LD		A,(.winulx)
	LD		D,A
	LD		A,(.winlrx)
	SUB		D
	INC		A
	LD		D,A				; D = window width in characters

2$:
	LDH		A,(.LCDSTAT)
	AND		#0x02
	JR		NZ,2$

	LD		A,(HL+)
	LD		(BC),A
	INC		BC
	DEC		D
	JR		NZ,2$

	POP		HL

	LD		BC,#0x20
	ADD		HL,BC
	PUSH	HL
	LD		BC,#0x20
	ADD		HL,BC		;HL=next line
	POP		BC

	DEC		E
	JR		NZ,1$

	; Fill new line with all spaces
	LD		A,(.winulx)
	LD		D,A
	LD		A,(.winlrx)
	SUB		D
	INC		A
	LD		D,A				; D = window width in characters

3$:
	LDH		A,(.LCDSTAT)
	AND		#0x02
	JR		NZ,3$

	LD		A,#0x20
	LD		(BC),A
	INC		BC
	DEC		D
	JR		NZ,3$

	RET

	; Set Cursor Window
	; BC = X,Y upper left of window
	; DE = X,Y lower right of window
.window:
	LD		A,B
	DEC		A
	LD		(.winulx),A
	LD		A,C
	DEC		A
	LD		(.winuly),A
	LD		A,D
	DEC		A
	LD		(.winlrx),A
	LD		A,E
	DEC		A
	LD		(.winlry),A
	RET

	; Print string pointed to by BC and terminated with a 0
.prntstr:
	LD		A,(BC)
	OR		A
	RET		Z
	CALL	.outchar
	INC		BC
	JR		.prntstr

	; Print a Space
.space:
	LD		A,#' '
	JP		.outchar

; Print a CR & LF
.crlf:
	LD	A,#0x0D

; Print a Character & protect all registers

.outchar::
	PUSH	AF
	PUSH	BC
	PUSH	DE
	PUSH	HL
	CALL	._outchar
	POP	HL
	POP	DE
	POP	BC
	POP	AF
	RET

	; Print a Character and update cursor position
._outchar:
	CP		#.BS			; Is it backspace ?
	JR		Z,8$			; Yes

	CP		#.CR			; Is it enter ?
	JR		NZ,1$			; No

	LD		A,#' '
	CALL	.outch			; Get rid of cursor
	CALL	5$				; Move cursor down a line
	JR		2$				; Display cursor
	
1$:
	CALL	.outch
	CALL	3$
2$:
	LD		A,#'_'
	JP		.outch

	; Update Cursor position
3$:
	LD		A,(.winulx)
	LD		B,A
	LD		A,(.winlrx)
	SUB		B
	INC		A
	LD		B,A				; B = window width
	LD		A,(.curx)
	CP		B				; Is cursor at end of line ?
	JR		Z,5$			; Yes

4$:
	INC		A
	LD		(.curx),A
	RET

5$:
	LD		A,(.winuly)
	LD		B,A
	LD		A,(.winlry)
	SUB		B
	INC		A
	LD		B,A				; B = window width
	LD		A,(.cury)
	CP		B				; Is cursor on last line ?
	JR		Z,7$			; Yes

	LD		A,(.cury)
	INC		A
	LD		(.cury),A

6$:
	XOR		A
	JR		4$

7$:
	CALL	.scroll
	JR		6$

8$:
	LD		A,(.curx)
	CP		#0x01			; Are we at beginning ?
	RET		Z				; Yes, do nothing

	PUSH	AF
	LD		A,#' '
	CALL	.outch
	POP		AF
	DEC		A
	LD		(.curx),A
	LD		A,#'_'
	JP		.outch

	; HighLight
.highlight:
	LD		A,#0x01
	LD		(.bright),A
	RET

	; LowLight
.lowlight:
	XOR		A
	LD		(.bright),A
	RET

	; Print a Character
.outch:
	PUSH	AF

	LD		A,(.winulx)
	LD		D,A
	LD		A,(.curx)
	DEC		A
	ADD		A,D
	LD		D,A

	LD		A,(.winuly)
	LD		E,A
	LD		A,(.cury)
	DEC		A
	ADD		A,E
	LD		E,A

	LD		L,E
	LD		H,#0x00
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	ADD		HL,HL
	LD		C,D
	LD		B,#0x00
	ADD		HL,BC
	LD		BC,#0x9800
	ADD		HL,BC

	LD		A,(.bright)		; Bright if 1
	LD		B,A
	POP		AF
	AND		#0x7F
	DEC		B
	JR		Z,1$

	ADD		A,#0x80
1$:
	LD		B,A

2$:
	LDH		A,(.LCDSTAT)
	AND		#0x02
	JR		NZ,2$
	
	LD		(HL),B
	RET

.getinbuf:
	LD		HL,#.inbuf
.getline::
	PUSH	HL
	CALL	.input			; Input a line from user
	POP		HL
	LD		A,(HL)			; Get first byte in buffer
	CP		#0x00			; Was it a blank line ?
	JR		Z,.getline		; Yes, don't do anything
	RET

	; Input a line of data at inbuf ending with a 0
.input:
	LD		B,#0x00
1$:
	PUSH	BC
	PUSH	HL
	CALL	.getch
	POP		HL
	POP		BC
	CP		#.BS			; Is it a backspace ?
	JR		Z,2$			; Yes
	LD		(HL),A			; Save character
	CP		#.CR			; Is it a cr ?
	JR		Z,3$			; Yes
	INC		HL
	INC		B
	CALL	.outchar
	JR		1$
2$:
	LD		A,B
	OR		A				; Are there any characters in buffer ?
	JR		Z,1$			; No, so don't delete

	DEC		B
	DEC		HL
	LD		A,#.BS			; Print a delete
	CALL	.outchar
	JR		1$
3$:
	LD		(HL),#0x00
	JP		.crlf

	; Convert A to lower case
.cnvtlc:
	CP		#'A'
	JR		C,7$
	CP		#'Z'+1
	JR		NC,7$
	ADD		A,#0x20
7$:
	RET

	; Print hex number in HL
hexdbl:
	LD		A,H
	CALL	.hexout
	LD		A,L

	; Print hex number in A
.hexout:
	PUSH	AF
	PUSH	AF
	RRCA	
	RRCA	
	RRCA	
	RRCA	
	AND		#0x0F
	ADD		A,#'0'
	CP		#'9'+1
	JR		C,1$
	ADD		A,#0x07
1$:
	CALL	.outchar
	POP		AF
	AND		#0x0F
	ADD		A,#'0'
	CP		#'9'+1
	JR		C,2$
	ADD		A,#0x0F
2$:
	CALL	.outchar
	POP		AF
	RET

	; Get a decimal value of a hex digit
	; Exit: set carry if bad
.dval:
	CP		#'0'			; Is value to low ?
	RET		C				; Yes
	CP		#'9'+1			; Is it a number ?
	JR		NC,1$			; No
	SUB		#'0'
	RET
1$:
	CP		#'A'			; Is value to low ?
	RET		C				; Yes
	CP		#'F'+1			; Is value to high ?
	CCF
	RET		C				; Yes
	SUB		#'A'-10
	RET

	; Get single byte hex value
.gethx:	PUSH	AF
	LD		A,(DE)
	INC		DE
	CALL	.dval
	JR		C,1$			; Digit was bad
	RLCA
	RLCA
	RLCA
	RLCA
	LD		C,A				; Save in C
	LD		A,(DE)
	INC		DE
	CALL	.dval
	JR		C,1$			; Digit was bad
	ADD		A,C
	LD		C,A				; Put composite in C
	POP		AF
	OR		A				; Clear carry
	RET

1$:	POP		AF
	OR		A
	CCF
	RET

	; C = HEX Byte
.getshx:
	CALL	.gethx
	RET

	; BC = HEX WORD
.getdhx:
	CALL	.gethx
	JR		C,1$
	LD		B,C
	CALL	.gethx
1$:
	RET

; Convert everything in inbuf to uppercase

.upcbuf:
	PUSH	HL
	LD		HL,#.inbuf
1$:
	LD		A,(HL)
	CP		#.CR
	JR		Z,3$
	CP		#'A'
	JR		C,2$
	CP		#'Z'+1
	JR		NC,2$
	SUB		#0x20
	LD		(HL),A
2$:
	INC		HL
	JR		1$
3$:
	POP		HL
	RET

	.area	_DATA

.chset:

; 0x00
	.byte	0x00,0x00,0x00,0x1F,0x16,0x16,0x16,0x16			; upper left of box
	.byte	0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x00			; upper middle of box
	.byte	0x00,0x00,0x00,0xF8,0x08,0x08,0x08,0x08			; upper right of box
	.byte	0x16,0x16,0x16,0x16,0x16,0x16,0x16,0x16			; left middle of box
	.byte	0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08			; right middle of box
	.byte	0x10,0x10,0x10,0x10,0x1F,0x00,0x00,0x00			; lower left of box
	.byte	0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00			; lower middle of box
	.byte	0x08,0x08,0x08,0x08,0xF8,0x00,0x00,0x00			; lower right of box
	.byte	0xFF,0xFF,0xE7,0xC3,0xC3,0xE7,0xFF,0xFF
	.byte	0x00,0x3C,0x66,0x42,0x42,0x66,0x3C,0x00
	.byte	0x7E,0x42,0x42,0x42,0x42,0x42,0x42,0x7E
	.byte	0x0F,0x07,0x0F,0x7D,0xCC,0xCC,0xCC,0x78
	.byte	0x3C,0x66,0x66,0x66,0x3C,0x18,0x7E,0x18
	.byte	0x3F,0x33,0x3F,0x30,0x30,0x70,0xF0,0xE0
	.byte	0x7F,0x63,0x7F,0x63,0x63,0x67,0xE6,0xC0
	.byte	0x99,0x5A,0x3C,0xE7,0xE7,0x3C,0x5A,0x99

; 0x10
	.byte	0x80,0xE0,0xF8,0xFE,0xF8,0xE0,0x80,0x00
	.byte	0x02,0x0E,0x3E,0xFE,0x3E,0x0E,0x02,0x00
	.byte	0x18,0x3C,0x7E,0x18,0x18,0x7E,0x3C,0x18
	.byte	0x66,0x66,0x66,0x66,0x66,0x00,0x66,0x00
	.byte	0x7F,0xDB,0xDB,0x7B,0x1B,0x1B,0x1B,0x00
	.byte	0x3E,0x63,0x38,0x6C,0x6C,0x38,0xCC,0x78
	.byte	0x00,0x00,0x00,0x00,0x7E,0x7E,0x7E,0x00
	.byte	0x18,0x3C,0x7E,0x18,0x7E,0x3C,0x18,0xFF
	.byte	0x18,0x3C,0x7E,0x18,0x18,0x18,0x18,0x00
	.byte	0x18,0x18,0x18,0x18,0x7E,0x3C,0x18,0x00
	.byte	0x00,0x18,0x0C,0xFE,0x0C,0x18,0x00,0x00
	.byte	0x00,0x30,0x60,0xFE,0x60,0x30,0x00,0x00
	.byte	0x00,0x00,0xC0,0xC0,0xC0,0xFE,0x00,0x00
	.byte	0x00,0x24,0x66,0xFF,0x66,0x24,0x00,0x00
	.byte	0x00,0x18,0x3C,0x7E,0xFF,0xFF,0x00,0x00
	.byte	0x00,0xFF,0xFF,0x7E,0x3C,0x18,0x00,0x00

; 0x20
	.byte	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00			; space
	.byte	0x30,0x78,0x78,0x78,0x30,0x00,0x30,0x00
	.byte	0x6C,0x6C,0x6C,0x00,0x00,0x00,0x00,0x00
	.byte	0x6C,0x6C,0xFE,0x6C,0xFE,0x6C,0x6C,0x00
	.byte	0x30,0x7C,0xC0,0x78,0x0C,0xF8,0x30,0x00
	.byte	0x00,0xC6,0xCC,0x18,0x30,0x66,0xC6,0x00
	.byte	0x38,0x6C,0x38,0x76,0xDC,0xCC,0x76,0x00
	.byte	0x60,0x60,0xC0,0x00,0x00,0x00,0x00,0x00
	.byte	0x18,0x30,0x60,0x60,0x60,0x30,0x18,0x00
	.byte	0x60,0x30,0x18,0x18,0x18,0x30,0x60,0x00
	.byte	0x00,0x66,0x3C,0xFF,0x3C,0x66,0x00,0x00
	.byte	0x00,0x30,0x30,0xFC,0x30,0x30,0x00,0x00
	.byte	0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x60
	.byte	0x00,0x00,0x00,0xFC,0x00,0x00,0x00,0x00
	.byte	0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x00
	.byte	0x06,0x0C,0x18,0x30,0x60,0xC0,0x80,0x00

; 0x30
	.byte	0x7C,0xC6,0xCE,0xDE,0xF6,0xE6,0x7C,0x00
	.byte	0x30,0x70,0x30,0x30,0x30,0x30,0xFC,0x00
	.byte	0x78,0xCC,0x0C,0x38,0x60,0xCC,0xFC,0x00
	.byte	0x78,0xCC,0x0C,0x38,0x0C,0xCC,0x78,0x00
	.byte	0x1C,0x3C,0x6C,0xCC,0xFE,0x0C,0x1E,0x00
	.byte	0xFC,0xC0,0xF8,0x0C,0x0C,0xCC,0x78,0x00
	.byte	0x38,0x60,0xC0,0xF8,0xCC,0xCC,0x78,0x00
	.byte	0xFC,0xCC,0x0C,0x18,0x30,0x30,0x30,0x00
	.byte	0x78,0xCC,0xCC,0x78,0xCC,0xCC,0x78,0x00
	.byte	0x78,0xCC,0xCC,0x7C,0x0C,0x18,0x70,0x00
	.byte	0x00,0x30,0x30,0x00,0x00,0x30,0x30,0x00
	.byte	0x00,0x30,0x30,0x00,0x00,0x30,0x30,0x60
	.byte	0x18,0x30,0x60,0xC0,0x60,0x30,0x18,0x00
	.byte	0x00,0x00,0xFC,0x00,0x00,0xFC,0x00,0x00
	.byte	0x60,0x30,0x18,0x0C,0x18,0x30,0x60,0x00
	.byte	0x78,0xCC,0x0C,0x18,0x30,0x00,0x30,0x00

; 0x40
	.byte	0x7C,0xC6,0xDE,0xDE,0xDE,0xC0,0x78,0x00
	.byte	0x30,0x78,0xCC,0xCC,0xFC,0xCC,0xCC,0x00
	.byte	0xFC,0x66,0x66,0x7C,0x66,0x66,0xFC,0x00
	.byte	0x3C,0x66,0xC0,0xC0,0xC0,0x66,0x3C,0x00
	.byte	0xF8,0x6C,0x66,0x66,0x66,0x6C,0xF8,0x00
	.byte	0x7E,0x60,0x60,0x78,0x60,0x60,0x7E,0x00
	.byte	0x7E,0x60,0x60,0x78,0x60,0x60,0x60,0x00
	.byte	0x3C,0x66,0xC0,0xC0,0xCE,0x66,0x3E,0x00
	.byte	0xCC,0xCC,0xCC,0xFC,0xCC,0xCC,0xCC,0x00
	.byte	0x78,0x30,0x30,0x30,0x30,0x30,0x78,0x00
	.byte	0x1E,0x0C,0x0C,0x0C,0xCC,0xCC,0x78,0x00
	.byte	0xE6,0x66,0x6C,0x78,0x6C,0x66,0xE6,0x00
	.byte	0x60,0x60,0x60,0x60,0x60,0x60,0x7E,0x00
	.byte	0xC6,0xEE,0xFE,0xFE,0xD6,0xC6,0xC6,0x00
	.byte	0xC6,0xE6,0xF6,0xDE,0xCE,0xC6,0xC6,0x00
	.byte	0x38,0x6C,0xC6,0xC6,0xC6,0x6C,0x38,0x00

; 0x50
	.byte	0xFC,0x66,0x66,0x7C,0x60,0x60,0xF0,0x00
	.byte	0x78,0xCC,0xCC,0xCC,0xDC,0x78,0x1C,0x00
	.byte	0xFC,0x66,0x66,0x7C,0x6C,0x66,0xE6,0x00
	.byte	0x78,0xCC,0xE0,0x70,0x1C,0xCC,0x78,0x00
	.byte	0xFC,0x30,0x30,0x30,0x30,0x30,0x30,0x00
	.byte	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xFC,0x00
	.byte	0xCC,0xCC,0xCC,0xCC,0xCC,0x78,0x30,0x00
	.byte	0xC6,0xC6,0xC6,0xD6,0xFE,0xEE,0xC6,0x00
	.byte	0xC6,0xC6,0x6C,0x38,0x38,0x6C,0xC6,0x00
	.byte	0xCC,0xCC,0xCC,0x78,0x30,0x30,0x78,0x00
	.byte	0xFE,0x06,0x0C,0x18,0x30,0x60,0xFE,0x00
	.byte	0x78,0x60,0x60,0x60,0x60,0x60,0x78,0x00
	.byte	0xC0,0x60,0x30,0x18,0x0C,0x06,0x02,0x00
	.byte	0x78,0x18,0x18,0x18,0x18,0x18,0x78,0x00
	.byte	0x10,0x38,0x6C,0xC6,0x00,0x00,0x00,0x00
	.byte	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF

; 0x60
	.byte	0x30,0x30,0x18,0x00,0x00,0x00,0x00,0x00
	.byte	0x00,0x00,0x78,0x0C,0x7C,0xCC,0x76,0x00
	.byte	0xE0,0x60,0x60,0x7C,0x66,0x66,0xDC,0x00
	.byte	0x00,0x00,0x78,0xCC,0xC0,0xCC,0x78,0x00
	.byte	0x1C,0x0C,0x0C,0x7C,0xCC,0xCC,0x76,0x00
	.byte	0x00,0x00,0x78,0xCC,0xFC,0xC0,0x78,0x00
	.byte	0x38,0x6C,0x60,0xF0,0x60,0x60,0xF0,0x00
	.byte	0x00,0x00,0x76,0xCC,0xCC,0x7C,0x0C,0xF8
	.byte	0xE0,0x60,0x6C,0x76,0x66,0x66,0xE6,0x00
	.byte	0x30,0x00,0x70,0x30,0x30,0x30,0x78,0x00
	.byte	0x0C,0x00,0x0C,0x0C,0x0C,0xCC,0xCC,0x78
	.byte	0xE0,0x60,0x66,0x6C,0x78,0x6C,0xE6,0x00
	.byte	0x70,0x30,0x30,0x30,0x30,0x30,0x78,0x00
	.byte	0x00,0x00,0xCC,0xFE,0xFE,0xD6,0xC6,0x00
	.byte	0x00,0x00,0xF8,0xCC,0xCC,0xCC,0xCC,0x00
	.byte	0x00,0x00,0x78,0xCC,0xCC,0xCC,0x78,0x00

; 0x70
	.byte	0x00,0x00,0xDC,0x66,0x66,0x7C,0x60,0xF0
	.byte	0x00,0x00,0x76,0xCC,0xCC,0x7C,0x0C,0x1E
	.byte	0x00,0x00,0xDC,0x76,0x66,0x60,0xF0,0x00
	.byte	0x00,0x00,0x7C,0xC0,0x78,0x0C,0xF8,0x00
	.byte	0x10,0x30,0x7C,0x30,0x30,0x34,0x18,0x00
	.byte	0x00,0x00,0xCC,0xCC,0xCC,0xCC,0x76,0x00
	.byte	0x00,0x00,0xCC,0xCC,0xCC,0x78,0x30,0x00
	.byte	0x00,0x00,0xC6,0xD6,0xFE,0xFE,0x6C,0x00
	.byte	0x00,0x00,0xC6,0x6C,0x38,0x6C,0xC6,0x00
	.byte	0x00,0x00,0xCC,0xCC,0xCC,0x7C,0x0C,0xF8
	.byte	0x00,0x00,0xFC,0x98,0x30,0x64,0xFC,0x00
	.byte	0x1C,0x30,0x30,0xE0,0x30,0x30,0x1C,0x00
	.byte	0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00
	.byte	0xE0,0x30,0x30,0x1C,0x30,0x30,0xE0,0x00
	.byte	0x76,0xDC,0x00,0x00,0x00,0x00,0x00,0x00
	.byte	0x00,0x10,0x38,0x6C,0xC6,0xC6,0xFE,0x00

.splash:
	.ascii	"     GB Runtime                 "
	.ascii	"                                "
	.ascii	"     Terminal &                 "
	.ascii	"   Basic Graphics               "
	.ascii	"                                "
	.ascii	"   Character Set:               "
	.ascii	"                                "

	.byte	0x20,0x20,0x00,0x01,0x02,0x03,0x04,0x05
	.byte	0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D
	.byte	0x0E,0x0F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x10,0x11,0x12,0x13,0x14,0x15
	.byte	0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D
	.byte	0x1E,0x1F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x20,0x21,0x22,0x23,0x24,0x25
	.byte	0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D
	.byte	0x2E,0x2F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x30,0x31,0x32,0x33,0x34,0x35
	.byte	0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D
	.byte	0x3E,0x3F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x40,0x41,0x42,0x43,0x44,0x45
	.byte	0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D
	.byte	0x4E,0x4F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x50,0x51,0x52,0x53,0x54,0x55
	.byte	0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D
	.byte	0x5E,0x5F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x60,0x61,0x62,0x63,0x64,0x65
	.byte	0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D
	.byte	0x6E,0x6F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.byte	0x20,0x20,0x70,0x71,0x72,0x73,0x74,0x75
	.byte	0x76,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7D
	.byte	0x7E,0x7F,0x20,0x20,0x20,0x20,0x20,0x20
	.byte	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20

	.ascii	"                                "
	.ascii	"   [Press START!]               "
	.ascii	"                                "
